@@ -334,7 +334,7 @@ typedef struct {
 
 /* To verify the rendering visually, use
  *
- * hb-view --font-slant SLANT --font-palette PALETTE FONT --glyphs [gidGID=0+1000]
+ * hb-view --font-slant SLANT --font-palette PALETTE FONT --glyphs gidGID
  *
  * where GID is the glyph value of the test.
  */
@@ -404,17 +404,19 @@ test_hb_paint (gconstpointer d,
 
   /* Run
    *
-   * GENERATE_DATA=1 G_TEST_SRCDIR=./test/api ./build/test/api/test-paint -p TESTCASE > test/api/results/OUTPUT
+   * GENERATE_DATA=1 G_TEST_SRCDIR=./test/api ./build/test/api/test-paint
    *
-   * to produce the expected results file.
+   * to regenerate the expected output for all tests.
    */
+  file = g_test_build_filename (G_TEST_DIST, "results-paint", test->output, NULL);
   if (getenv ("GENERATE_DATA"))
     {
-      g_print ("%s", data.string->str);
-      exit (0);
+      if (!g_file_set_contents (file, data.string->str, data.string->len, NULL))
+        g_error ("Failed to write %s.", file);
+
+      return;
     }
 
-  file = g_test_build_filename (G_TEST_DIST, "results", test->output, NULL);
   if (!g_file_get_contents (file, &buffer, &len, &error))
   {
     g_test_message ("File %s not found.", file);
@@ -429,22 +431,6 @@ test_hb_paint (gconstpointer d,
   else
     expected = g_strsplit (buffer, "\n", 0);
 
-  /* Strip initial comments */
-  int i;
-  for (i = 0; expected[i]; i++)
-    {
-      if (expected[i][0] != '#')
-        {
-          if (i > 0)
-            {
-              char **tmp = g_strdupv (expected + i);
-              g_strfreev (expected);
-              expected = tmp;
-            }
-          break;
-        }
-    }
-
   if (g_strv_length (lines) != g_strv_length (expected))
   {
     g_test_message ("Unexpected number of lines in output (%d instead of %d):\n%s", g_strv_length (lines), g_strv_length (expected), data.string->str);
@@ -481,7 +467,7 @@ test_hb_paint (gconstpointer d,
 }
 
 static void
-test_compare_ot_ft (const char *file, hb_codepoint_t glyph)
+test_compare_to_ot (const char *file, hb_codepoint_t glyph)
 {
   hb_face_t *face;
   hb_font_t *font;
@@ -539,21 +525,15 @@ test_hb_paint_ft (gconstpointer data)
 }
 
 static void
-test_compare_ot_ft_novf (gconstpointer d)
+test_compare_to_ot_novf (gconstpointer d)
 {
-  if (have_ft_colrv1 ())
-    test_compare_ot_ft (TEST_GLYPHS, GPOINTER_TO_UINT (d));
-  else
-    g_test_skip ("FreeType COLRv1 support not present");
+  test_compare_to_ot (TEST_GLYPHS, GPOINTER_TO_UINT (d));
 }
 
 static void
-test_compare_ot_ft_vf (gconstpointer d)
+test_compare_to_ot_vf (gconstpointer d)
 {
-  if (have_ft_colrv1 ())
-    test_compare_ot_ft (TEST_GLYPHS_VF, GPOINTER_TO_UINT (d));
-  else
-    g_test_skip ("FreeType COLRv1 support not present");
+  test_compare_to_ot (TEST_GLYPHS_VF, GPOINTER_TO_UINT (d));
 }
 
 static void
@@ -664,12 +644,20 @@ main (int argc, char **argv)
 
   hb_face_t *face = hb_test_open_font_file (TEST_GLYPHS);
   unsigned glyph_count = hb_face_get_glyph_count (face);
-  for (unsigned int i = 1; i < glyph_count; i++)
+  const char **font_funcs = hb_font_list_funcs ();
+  for (const char **font_func = font_funcs; *font_func; font_func++)
   {
-    char buf[20];
-    snprintf (buf, 20, "test-%u", i);
-    hb_test_add_data_flavor (GUINT_TO_POINTER (i), buf, test_compare_ot_ft_novf);
-    hb_test_add_data_flavor (GUINT_TO_POINTER (i), buf, test_compare_ot_ft_vf);
+    if (strcmp (*font_func, "ot") == 0)
+      continue;
+    if (!have_ft_colrv1 () && strcmp (*font_func, "ft") == 0)
+      continue;
+    for (unsigned int i = 1; i < glyph_count; i++)
+    {
+      char buf[32];
+      snprintf (buf, 32, "test-%s-%u", *font_func, i);
+      hb_test_add_data_flavor (GUINT_TO_POINTER (i), buf, test_compare_to_ot_novf);
+      hb_test_add_data_flavor (GUINT_TO_POINTER (i), buf, test_compare_to_ot_vf);
+    }
   }
   hb_face_destroy (face);
 
